% 1 - INITIALIZE UQLAB
clearvars
rng(1,'twister')
uqlab

% 2 - COMPUTATIONAL MODEL
% Create a MODEL object for the constraint (limit state) function model 
% using the function file:
ModelOpts.mFile = 'uq_beam_constraint';
myModel = uq_createModel(ModelOpts);

% 3 - PROBABILISTIC INPUT MODEL
% The environmental variables are modeled probalistically using three
% independent random variables.
% Specify the marginals each of which follows a lognormal distribution:
%InputOpts.Marginals(1).Name = 'k' ;  % the multiplicative parameter
%InputOpts.Marginals(1).Type = 'Lognormal';
%InputOpts.Marginals(1).Moments = [0.6 0.1*0.6];

InputOpts.Marginals(1).Name = 'E';  % Young's modulus of the column
InputOpts.Marginals(1).Type = 'Lognormal';
InputOpts.Marginals(1).Moments = [7.2e4 0.05*7.2e4];

InputOpts.Marginals(2).Name = 'L';  % length of the column
InputOpts.Marginals(2).Type = 'Lognormal';
InputOpts.Marginals(2).Moments = [2e3 0.01*2e3];

% Create an INPUT object based on the specified marginals:
myInput = uq_createInput(InputOpts);

% 4 - RELIABILITY-BASED DESIGN OPTIMIZATION (RBDO) SETUP
% 4.1 Design and environmental variables
% The probabilistic input model consists of two deterministic design
% parameters and three random environmental variables.
% Specify the design variables:
RBDOOpts.Input.DesVar(1).Name = 'b';
RBDOOpts.Input.DesVar(1).Type = 'constant';
RBDOOpts.Input.DesVar(2).Name = 'h';
RBDOOpts.Input.DesVar(2).Type = 'constant';

% Specify the INPUT object used for the environmental variables:
RBDOOpts.Input.EnvVar = myInput;

% 4.2 Cost function
% The cost function reads:
% Create a cost function model using a Matlab string (X(1) is b and X(2) is h):
RBDOOpts.Cost.mString = 'X(:,1) .* X(:,2)';

% 4.3 Hard constraint
% The limit state function corresponds to the predefined computational
% model. 
% Assign this to the RBDO analysis options:
RBDOOpts.LimitState.Model = myModel;

% 4.4 Soft constraint
% Create a soft constraint function using a string (understood as X(2)-X(1)>0)
RBDOOpts.SoftConstraints.mString = 'X(:,1) - X(:,2)';

% 4.5 Optimization setting
% To specify the optimization problem,
% the bounds of the design space are first defined as follows:
RBDOOpts.Optim.Bounds = [150 150; 350 350];

% Optionally, the starting point for the optimization algorithm
% can also be provided:
RBDOOpts.Optim.StartingPoint = [300 300];

% Then, define the target failure probability:
RBDOOpts.TargetPf = 0.05;

% 5 - RELIABILITY-BASED DESIGN OPTIMIZATION (RBDO)
% 5.1 Reliability index approach (RIA)
% Select RIA to solve the RBDO problem:
RIAOpts = RBDOOpts;
RIAOpts.Type = 'RBDO';
RIAOpts.Method = 'RIA';

% Run the RBDO analysis:
myRBDO_RIA = uq_createAnalysis(RIAOpts) ;
% Print out a report of the results:
uq_print(myRBDO_RIA)
% Display a graphical representation of the results:
uq_display(myRBDO_RIA)

% 5.2 Performance measure approach (PMA)
% Select PMA to solve the RBDO problem:
PMAOpts = RBDOOpts;
PMAOpts.Type = 'RBDO';
PMAOpts.Method = 'PMA';
% Run the RBDO analysis:
myRBDO_PMA = uq_createAnalysis(PMAOpts) ;

% Print out a report of the results:
uq_print(myRBDO_PMA)
% Display a graphical representation of the results:
uq_display(myRBDO_PMA)

% 5.3 Sequential optimization and reliability assessment (SORA)
% Select SORA to solve the RBDO problem:
SORAOpts = RBDOOpts;
SORAOpts.Type = 'RBDO';
SORAOpts.Method = 'SORA';
% Run the RBDO analysis:
myRBDO_SORA = uq_createAnalysis(SORAOpts) ;
% Print out a report of the results:
uq_print(myRBDO_SORA)
% Display a graphical representation of the results:
uq_display(myRBDO_SORA)

% 5.4 Single loop approach (SLA)
% Select SLA to solve the RBDO problem:
SLAOpts = RBDOOpts;
SLAOpts.Type = 'RBDO';
SLAOpts.Method = 'SLA';
% Run the RBDO analysis:
myRBDO_SLA = uq_createAnalysis(SLAOpts);
% Print out a report of the results:
uq_print(myRBDO_SLA)
% Display a graphical representation of the results:
uq_display(myRBDO_SLA)

% 5.5 Two-level approach with Monte Carlo simulation
% Select the generalized two-level approach to solve the RBDO problem: 
TLOpts = RBDOOpts ;
TLOpts.Type = 'RBDO';
TLOpts.Method = 'two-level';

% Run the RBDO analysis:
myRBDO_TL = uq_createAnalysis(TLOpts);
% Print out a report of the results:
uq_print(myRBDO_TL)
% Create a graphical representation of the results:
uq_display(myRBDO_TL)

%% 5.6 Quantile Monte Carlo (QMC) approach
%
% Select QMC as the RBDO method:
QMCOpts = RBDOOpts;
QMCOpts.Type = 'RBDO';
QMCOpts.Method = 'QMC';

% Run the RBDO analysis:
myRBDO_QMC = uq_createAnalysis(QMCOpts) ;
% Print out a report of the results:
uq_print(myRBDO_QMC)
% Display a graphical representation of the results:
uq_display(myRBDO_QMC)
