% 1 - INITIALIZE UQLAB
clearvars
rng(1,'twister')
uqlab

% 2 - COMPUTATIONAL MODEL
% Create a MODEL object for the constraint (limit state) function model 
% using the function file:
ModelOpts.Type = 'UQLink';
ModelOpts.Name = 'Beam_Deflection';
ModelOpts.Command = 'abaqus cae noGui=Beam_Deflection.py';
ModelOpts.Template = 'Beam_Deflection.py.tpl';
ModelOpts.Output.FileName = 'abaqus.rpt';
ModelOpts.Output.Parser = 'readabaqusOutput';
ModelOpts.ExecutionPath = fullfile('D:\',...
    'Beam_Deflection') ;
ModelOpts.Format = {'%.6f'};
ModelOpts.Display = 'quiet';
myUQLinkModel = uq_createModel(ModelOpts);

% 3 - PROBABILISTIC INPUT MODEL
% The environmental variables are modeled probalistically using three
% independent random variables.
% Specify the marginals each of which follows a lognormal distribution:
%InputOpts.Marginals(1).Name = 'k' ;  % the multiplicative parameter
%InputOpts.Marginals(1).Type = 'Lognormal';
%InputOpts.Marginals(1).Moments = [0.6 0.1*0.6];

InputOpts.Marginals(1).Name = 'E';  % Young's modulus of the column
InputOpts.Marginals(1).Type = 'Lognormal';
InputOpts.Marginals(1).Moments = [7.2e4 0.05*7.2e4];

InputOpts.Marginals(2).Name = 'L';  % length of the column
InputOpts.Marginals(2).Type = 'Lognormal';
InputOpts.Marginals(2).Moments = [2e3 0.01*2e3];

% Create an INPUT object based on the specified marginals:
myInput = uq_createInput(InputOpts);

% 4 - RELIABILITY-BASED DESIGN OPTIMIZATION (RBDO) SETUP
% 4.1 Design and environmental variables
% The probabilistic input model consists of two deterministic design
% parameters and three random environmental variables.
% Specify the design variables:
RBDOOpts.Input.DesVar(1).Name = 'b';
RBDOOpts.Input.DesVar(1).Type = 'constant';
RBDOOpts.Input.DesVar(2).Name = 'h';
RBDOOpts.Input.DesVar(2).Type = 'constant';

% Specify the INPUT object used for the environmental variables:
RBDOOpts.Input.EnvVar = myInput;

% 4.2 Cost function
% The cost function reads:
% Create a cost function model using a Matlab string (X(1) is b and X(2) is h):
RBDOOpts.Cost.mString = 'X(:,1) .* X(:,2)';

% 4.3 Hard constraint
% The limit state function corresponds to the predefined computational
% model. 
% Assign this to the RBDO analysis options:
RBDOOpts.LimitState.Model = myUQLinkModel;

% 4.4 Soft constraint
% Create a soft constraint function using a string (understood as X(2)-X(1)>0)
RBDOOpts.SoftConstraints.mString = 'X(:,1) - X(:,2)';

% 4.5 Optimization setting
% To specify the optimization problem,
% the bounds of the design space are first defined as follows:
RBDOOpts.Optim.Bounds = [150 150; 350 350];

% Optionally, the starting point for the optimization algorithm
% can also be provided:
RBDOOpts.Optim.StartingPoint = [300 300];
% Set the maximum number of iterations:
RBDOOpts.Optim.MaxIter = 500;
% Then, define the target failure probability:
RBDOOpts.TargetPf = 0.05;

% 5 - RELIABILITY-BASED DESIGN OPTIMIZATION (RBDO)

% Select QMC as the RBDO method:
RBDOOpts.Type = 'RBDO';
RBDOOpts.Method = 'QMC';

% 4.3  Kriging
% Initialize the options for Kriging-assisted RBDO:
RBDO_KrgOpts = RBDOOpts;

% Select Kriging as the metamodel type for RBDO:
RBDO_KrgOpts.Metamodel.Type = 'Kriging';

% Set the options for the Kriging metamodel:
KrgOpts.EstimMethod = 'ML';
KrgOpts.Corr.Family = 'matern-5_2';
KrgOpts.Corr.Isotropic = false;
KrgOpts.Optim.Method = 'HGA';
KrgOpts.ExpDesign.NSamples = 10;

% Specify the options for the construction of the Kriging metamodel"
RBDO_KrgOpts.Metamodel.KRIGING = KrgOpts;

% Specify the enrichment options for the adaptive experimental design:
RBDO_KrgOpts.Metamodel.Enrichment.Convergence = {'stopSign','stopDf'};
RBDO_KrgOpts.Metamodel.Enrichment.ConvThreshold = 0.001;
RBDO_KrgOpts.Metamodel.Enrichment.MaxAdded = 50;
RBDO_KrgOpts.Metamodel.Enrichment.Points = 2;

% Specify SQP as the optimization algorithm:
RBDO_KrgOpts.Optim.Method = 'SQP';
 
% Run the RBDO analysis:
myRBDO_Krg = uq_createAnalysis(RBDO_KrgOpts);

% Print out a report of the results:
uq_print(myRBDO_Krg)

% Display a graphical representation of the results:
uq_display(myRBDO_Krg)



